@@ -133,9 +133,9 @@ void check_load_balancer(t_config *config) {
 							last->next = next;
 						}
 						free(cgi_session);
+					} else {
+						last = cgi_session;
 					}
-
-					last = cgi_session;
 					cgi_session = next;
 				}
 
@@ -314,7 +314,7 @@ pid_t fork_cgi_process(t_session *session, t_cgi_info *cgi_info) {
 	return cgi_pid;
 }
 
-int read_from_cgi_process(t_session *session, t_cgi_info *cgi_info) {
+t_cgi_result read_from_cgi_process(t_session *session, t_cgi_info *cgi_info) {
 	bool read_again;
 	fd_set read_fds;
 	int bytes_read, result = cgi_OKE;
@@ -447,7 +447,7 @@ int	send_fcgi_request(t_session *session, int sock) {
 	return 0;
 }
 
-static int read_fcgi_socket(t_session *session, t_cgi_info *cgi_info, char *buffer, int size) {
+static t_cgi_result read_fcgi_socket(t_session *session, t_cgi_info *cgi_info, char *buffer, int size) {
 	int bytes_read, result = cgi_ERROR;
 	fd_set read_fds;
 	bool read_again;
@@ -490,7 +490,7 @@ static int read_fcgi_socket(t_session *session, t_cgi_info *cgi_info, char *buff
 	return result;
 }
 
-int read_from_fcgi_server(t_session *session, t_cgi_info *cgi_info) {
+t_cgi_result read_from_fcgi_server(t_session *session, t_cgi_info *cgi_info) {
 	char *buffer, dummy[DUMMY_BUFFER_SIZE];
 	bool read_again;
 	int bytes_read, bytes_left;
