@@ -57,7 +57,7 @@ static int add_to_environment(t_fcgi_buffer *fcgi_buffer, char *key, char *value
 	return 0;
 }
 
-int add_to_environment_chroot(t_session *session, t_fcgi_buffer *fcgi_buffer, char *key, char *value) {
+static int add_to_environment_chroot(t_session *session, t_fcgi_buffer *fcgi_buffer, char *key, char *value) {
 	size_t ofs = 0;
 	char c;
 
@@ -102,8 +102,8 @@ void set_environment(t_session *session, t_fcgi_buffer *fcgi_buffer) {
 
 	add_to_environment(fcgi_buffer, "REQUEST_METHOD", session->method);
 	add_to_environment(fcgi_buffer, "REQUEST_URI", session->uri);
-	if (session->uri[strlen(session->uri) - 1] == '/') {
-		len1 = strlen(session->uri);
+	if (session->uri[session->uri_len - 1] == '/') {
+		len1 = session->uri_len;
 		len2 = strlen(session->host->start_file);
 		if ((name = (char*)malloc(len1 + len2 + 1)) != NULL) {
 			memcpy(name, session->uri, len1);
@@ -115,6 +115,9 @@ void set_environment(t_session *session, t_fcgi_buffer *fcgi_buffer) {
 	} else {
 		add_to_environment(fcgi_buffer, "SCRIPT_NAME", session->uri);
 	}
+#ifdef HAVE_REWRITE
+	add_to_environment(fcgi_buffer, "SCRIPT_URL", session->script_url);
+#endif
 
 	add_to_environment_chroot(session, fcgi_buffer, "SCRIPT_FILENAME", session->file_on_disk);
 #ifdef CYGWIN
