@@ -37,9 +37,7 @@
 #include "client.h"
 #include "target.h"
 #include "log.h"
-#ifndef HAVE_SETENV
 #include "envir.h"
-#endif
 #ifdef HAVE_COMMAND
 #include "command.h"
 #endif
@@ -49,6 +47,9 @@
 #ifdef HAVE_CACHE
 #include "cache.h"
 #endif
+#ifdef HAVE_REWRITE
+#include "rewrite.h"
+#endif
 
 #define REQUEST_BUFFER_CHUNK  4096
 #define MAIN_LOOP_TIMER          1
@@ -87,6 +88,9 @@ char *version_string = "Hiawatha v"VERSION
 #ifdef HAVE_CACHE
 			", cache"
 #endif
+#ifdef HAVE_REWRITE
+			", URL rewrite"
+#endif
 #ifdef HAVE_COMMAND
 			", CommandChannel"
 #endif
@@ -140,7 +144,7 @@ void touch_logfiles(t_config *config, char *dir) {
 
 /* Check if the requested file is a CGI program.
  */
-void check_target_is_cgi(t_session *session) {
+t_cgi_type check_target_is_cgi(t_session *session) {
 	t_cgi_handler *cgi;
 
 	if ((session->fcgi_server = fcgi_server_match(session->config->fcgi_server, &(session->host->fast_cgi), session->file_on_disk)) != NULL) {
@@ -158,35 +162,35 @@ void check_target_is_cgi(t_session *session) {
 			cgi = cgi->next;
 		}
 	}
+
+	return session->cgi_type;
 }
 
 /* Handle an HTTP error.
  */
 int handle_error(t_session *session, int error_code) {
-	char **error_handler, *no_handler = NULL;
-	int length;
+	char *error_handler = NULL;
+	int length, result = -1;
 
 	switch (error_code) {
 		case 401:
-			error_handler = &(session->host->error_401_handler);
+			error_handler = session->host->error_401_handler;
 			break;
 		case 403:
-			error_handler = &(session->host->error_403_handler);
+			error_handler = session->host->error_403_handler;
 			break;
 		case 404:
-			error_handler = &(session->host->error_404_handler);
+			error_handler = session->host->error_404_handler;
 			break;
 		case 501:
-			error_handler = &(session->host->error_501_handler);
+			error_handler = session->host->error_501_handler;
 			break;
 		case 503:
-			error_handler = &(session->host->error_503_handler);
+			error_handler = session->host->error_503_handler;
 			break;
-		default:
-			error_handler = &no_handler;
 	}
-	if (*error_handler == NULL) {
-		return error_code;
+	if (error_handler == NULL) {
+		return 0;
 	}
 
 	session->return_code = error_code;
@@ -196,23 +200,57 @@ int handle_error(t_session *session, int error_code) {
 	if (session->file_on_disk != NULL) {
 		free(session->file_on_disk);
 	}
-	length = strlen(session->host->website_root);
-	if ((session->file_on_disk = (char*)malloc(length + strlen(*error_handler) + 4)) == NULL) { // + 3 for .gz (gzip encoding)
-		return 500;
-	}
-	strcpy(session->file_on_disk, session->host->website_root);
-	strcpy(session->file_on_disk + length, *error_handler);
+	length = session->host->website_root_len;
+	if ((session->file_on_disk = (char*)malloc(length + strlen(error_handler) + 4)) == NULL) { // + 3 for .gz (gzip encoding)
+		result = 500;
+	} else {
+		memcpy(session->file_on_disk, session->host->website_root, length + 1);
+		strcpy(session->file_on_disk + length, error_handler);
 
-	session->cgi_type = none;
-	check_target_is_cgi(session);
+		session->cgi_type = none;
+		check_target_is_cgi(session);
 
-	if (session->cgi_type != none) {
-		return execute_cgi(session);
-	} else if (is_directory(session->file_on_disk) == no) {
-		return send_file(session);
+		if (session->cgi_type != none) {
+			result = execute_cgi(session);
+		} else switch (is_directory(session->file_on_disk)) {
+			case error:
+				result = 500;
+				break;
+			case yes:
+				result = 301;
+				break;
+			case no:
+				result = send_file(session);
+				break;
+			case no_access:
+				result = 403;
+				break;
+			case not_found:
+				result = 404;
+				break;
+		}
 	}
 
-	return error_code;
+	switch (result) {
+		case 301:
+			log_file_error(session, session->file_on_disk, "ErrorHandler is a directory");
+			break;
+		case 403:
+			log_file_error(session, session->file_on_disk, "no access to ErrorHandler");
+			break;
+		case 404:
+			log_file_error(session, session->file_on_disk, "ErrorHandler not found");
+			break;
+		case 500:
+			log_file_error(session, error_handler, "internal error for ErrorHandler");
+			session->keep_alive = false;
+			break;
+		case 503:
+			log_file_error(session, error_handler, "FastCGI for ErrorHandler not available");
+			break;
+	}
+
+	return result;
 }
 
 /* Handle the request from a client.
@@ -220,8 +258,6 @@ int handle_error(t_session *session, int error_code) {
 int handle_request(t_session *session) {
 	int retval = 200;
 
-	check_target_is_cgi(session);
-
 	if ((strcmp(session->method, "GET") == 0) || session->head_request) {
 		if (session->cgi_type != none) {
 			session->body = NULL;
@@ -230,7 +266,7 @@ int handle_request(t_session *session) {
 			retval = send_file(session);
 		}
 		if (retval == 404) {
-			if (session->host->show_index && (session->uri[strlen(session->uri) - 1] == '/')) {
+			if (session->host->show_index && (session->uri[session->uri_len - 1] == '/')) {
 				retval = show_index(session);
 			}
 		}
@@ -288,10 +324,11 @@ int handle_request(t_session *session) {
 int fetch_request(t_session *session) {
 	char *new_reqbuf, *strstart, *strend;
 	long bytes_read, header_length = -1, content_length = -1;
-	int result = 200, timer;
+	int result = 200, timer, write_bytes;
 	fd_set read_fds;
 	struct timeval select_timeout;
-	bool keep_reading = true;
+	bool keep_reading = true, store_on_disk = false;
+	int upload_handle = -1;
 
 	select_timeout.tv_sec = 1;
 	select_timeout.tv_usec = 0;
@@ -310,8 +347,36 @@ int fetch_request(t_session *session) {
 		if (session->request != NULL) {
 			if (header_length == -1) {
 				if ((strstart = strstr(session->request, "\r\n\r\n")) != NULL) {
+					*(strstart + 2) = '\0';
 					header_length = strstart + 4 - session->request;
 					session->header_length = header_length;
+
+					session->put_method = (strncmp(session->request, "PUT ", 4) == 0);
+					store_on_disk = (session->put_method && session->binding->enable_alter);
+
+					if (store_on_disk) {
+	 					if ((session->uploaded_file = (char*)malloc(session->config->upload_directory_len + 15)) != NULL) {
+							strcpy(session->uploaded_file, session->config->upload_directory);
+							strcpy(session->uploaded_file + session->config->upload_directory_len, "/upload_XXXXXX");
+							if ((upload_handle = mkstemp(session->uploaded_file)) == -1) {
+								free(session->uploaded_file);
+								session->uploaded_file = NULL;
+							}
+						}
+						if (session->uploaded_file == NULL) {
+							log_error(session, "can't create tempfile for PUT request");
+							result = 500;
+							break;
+						}
+
+						session->uploaded_size = session->bytes_in_buffer - header_length;
+						if (write_buffer(upload_handle, session->request + header_length, session->uploaded_size) == -1) {
+							result = 500;
+							break;
+						}
+						session->bytes_in_buffer = header_length;
+					}
+
 				}
 			}
 			if (header_length != -1) {
@@ -320,37 +385,63 @@ int fetch_request(t_session *session) {
 						strstart += 16;
 						if ((strend = strstr(strstart, "\r\n")) != NULL) {
 							*strend = '\0';
-							if ((content_length = str2int(strstart)) == -1) {
+							content_length = str2int(strstart);
+							*strend = '\r';
+							if (content_length == -1) {
 								result = 400;
 								break;
 							}
-							session->content_length = content_length;
-							*strend = '\r';
 
-							if (header_length + content_length > session->binding->max_request_size) {
-								keep_reading = false;
-								result = fe_MAX_REQUESTSIZE;
-								break;
-							}
+							if (store_on_disk) {
+								session->content_length = 0;
+								if (content_length > session->binding->max_upload_size) {
+									result = 413;
+									break;
+								}
 
-							if (header_length + content_length + 1 > session->buffer_size) {
-								session->buffer_size = header_length + content_length + 1;
-								if ((new_reqbuf = (char*)realloc(session->request, session->buffer_size)) != NULL) {
+								session->buffer_size = header_length + REQUEST_BUFFER_CHUNK;
+								if ((new_reqbuf = (char*)realloc(session->request, session->buffer_size + 1)) != NULL) {
 									session->request = new_reqbuf;
 								} else {
 									result = fe_READ_ERROR;
-									keep_reading = false;
 									break;
 								}
+							} else {
+								session->content_length = content_length;
+								if (header_length + content_length > session->binding->max_request_size) {
+									result = fe_MAX_REQUESTSIZE;
+									break;
+								}
+
+								if (header_length + content_length > session->buffer_size) {
+									session->buffer_size = header_length + content_length;
+									if ((new_reqbuf = (char*)realloc(session->request, session->buffer_size + 1)) != NULL) {
+										session->request = new_reqbuf;
+									} else {
+										result = fe_READ_ERROR;
+										break;
+									}
+								}
 							}
 						}
 					} else {
+						session->content_length = 0;
+						if (store_on_disk) {
+							result = 411;
+						}
 						break;
 					}
 				}
 				if (content_length > -1) {
-					if (session->bytes_in_buffer >= header_length + content_length) {
-						break;
+					if (store_on_disk) {
+						if (session->uploaded_size == content_length) {
+							break;
+						}
+					} else {
+						if (session->bytes_in_buffer >= header_length + content_length) {
+							// Received a complete request
+							break;
+						}
 					}
 				}
 			}
@@ -383,7 +474,7 @@ int fetch_request(t_session *session) {
 			default:
 				if ((content_length == -1) && ((session->buffer_size - session->bytes_in_buffer) < 256)) {
 					session->buffer_size += REQUEST_BUFFER_CHUNK;
-					if ((new_reqbuf = (char*)realloc(session->request, session->buffer_size)) != NULL) {
+					if ((new_reqbuf = (char*)realloc(session->request, session->buffer_size + 1)) != NULL) {
 						session->request = new_reqbuf;
 					} else {
 						result = fe_READ_ERROR;
@@ -400,11 +491,11 @@ int fetch_request(t_session *session) {
 #ifdef HAVE_SSL
 				if (session->binding->use_ssl) {
 					bytes_read = ssl_receive(session->ssl_data, session->request + session->bytes_in_buffer, 
-									session->buffer_size - session->bytes_in_buffer - 1);
+									session->buffer_size - session->bytes_in_buffer);
 				} else
 #endif
 					bytes_read = recv(session->client_socket, session->request + session->bytes_in_buffer,
-									session->buffer_size - session->bytes_in_buffer - 1, 0);
+									session->buffer_size - session->bytes_in_buffer, 0);
 
 				switch (bytes_read) {
 					case -1:
@@ -418,20 +509,48 @@ int fetch_request(t_session *session) {
 						keep_reading = false;
 						break;
 					default:
-						session->bytes_in_buffer += bytes_read;
-						*(session->request + session->bytes_in_buffer) = '\0';
+						if (store_on_disk) {
+							// Write to file on disk
+							write_bytes = bytes_read;
+							if (session->uploaded_size + bytes_read > content_length) {
+								write_bytes -= ((session->uploaded_size + bytes_read) - content_length);
+							}
+							if (write_buffer(upload_handle, session->request + header_length, write_bytes) == -1) {
+								result = 500;
+								keep_reading = false;
+								break;
+							}
+							if ((session->uploaded_size += write_bytes) > session->binding->max_upload_size) {
+								result = 413;
+								keep_reading = false;
+								break;
+							}
+							if (write_bytes < bytes_read) {
+								memmove(session->request + header_length, session->request + header_length + write_bytes, bytes_read - write_bytes);
+								session->bytes_in_buffer += bytes_read - write_bytes;
+								keep_reading = false;
+							}
+						} else {
+							// Read into memory
+							session->bytes_in_buffer += bytes_read;
+							*(session->request + session->bytes_in_buffer) = '\0';
 
-						if (session->bytes_in_buffer > session->binding->max_request_size) {
-							keep_reading = false;
-							result = fe_MAX_REQUESTSIZE;
-							break;
+							if (session->bytes_in_buffer > session->binding->max_request_size) {
+								keep_reading = false;
+								result = fe_MAX_REQUESTSIZE;
+								break;
+							}
 						}
 				}
 		}
 	} while (keep_reading);
 
+	if (upload_handle != -1) {
+		close(upload_handle);
+	}
+
 #ifdef HAVE_COMMAND
-	increment_transfer(TRANSFER_RECEIVED, session->header_length + session->content_length);
+	increment_transfer(TRANSFER_RECEIVED, header_length + content_length);
 #endif
 
 #ifdef DEBUG
@@ -465,23 +584,20 @@ int parse_request(t_session *session, int total_bytes) {
 	 */
 	while ((*str_end != ' ') && (str_end != request_end)) {
 		str_end++;
-		if (*str_end == '?') {
-			if (session->vars == NULL) {
-				session->vars = str_end + 1;
-				*str_end = '\0';
-			}
-		}
 	}
 	if (str_end == request_end) {
 		return 400;
 	}
-
-	*str_end = '\0';
-	str_end++;
+	*(str_end++) = '\0';
+#ifdef HAVE_REWRITE
+	if ((session->script_url = strdup(session->uri)) == NULL) {
+		return -1;
+	}
+#endif
 
 	/* Protocol version
 	 */
-	if (strlen(str_end) < 8) {
+	if (min_strlen(str_end, 10) == false) {
 		return 400;
 	} else if (memcmp(str_end, "HTTP/", 5) != 0) {
 		return 400;
@@ -490,16 +606,6 @@ int parse_request(t_session *session, int total_bytes) {
 	session->http_version = str_end;
 	str_end += 7;
 
-	/* Body and other request headerlines
-	 */
-	if (session->content_length > 0) {
-		session->body = session->request + session->header_length;
-		*(session->body - 2) = '\0';
-	}
-	if (str_end + 5 != session->body) {
-		session->headerfields = parse_headerfields(str_end + 3);
-	}
-
 	if ((*(str_end - 1) != '.') || (*(str_end + 1) != '\r') || (*(str_end + 2) != '\n')) {
 		return 400;
 	} else if (*(str_end - 2) != '1') {
@@ -507,6 +613,14 @@ int parse_request(t_session *session, int total_bytes) {
 	}
 	*(str_end + 1) = '\0';
 
+	/* Body and other request headerlines
+	 */
+	if (session->content_length > 0) {
+		session->body = session->request + session->header_length;
+	}
+	session->headerfields = parse_headerfields(str_end + 3);
+	session->hostname = get_headerfield("Host:", session->headerfields);
+
 	if ((conn = get_headerfield("Connection:", session->headerfields)) != NULL) {
 		conn = strlower(remove_spaces(conn));
 	}
@@ -520,7 +634,7 @@ int parse_request(t_session *session, int total_bytes) {
 			}
 			break;
 		case '1':
-			if (get_headerfield("Host:", session->headerfields) == NULL) {
+			if (session->hostname == NULL) {
 				retval = 400;
 			} else if (session->kept_alive < session->binding->max_keepalive) {
 				session->keep_alive = true;
@@ -556,7 +670,7 @@ int uri_to_path(t_session *session) {
 	
 	/* Requested file in userdirectory?
 	 */
-	if ((session->host->user_websites) && (strlen(session->uri) > 3)) {
+	if (session->host->user_websites && (session->uri_len >= 3)) {
 		if (*(session->uri + 1) == '~') {
 			strstart = session->uri + 1;
 			if ((strend = strchr(strstart, '/')) == NULL) {
@@ -582,7 +696,7 @@ int uri_to_path(t_session *session) {
 					return 500;
 				}
 			} else {
-				// uri is '/~/'
+				// uri is '/~/...'
 				return 404;
 			}
 		}
@@ -605,11 +719,11 @@ int uri_to_path(t_session *session) {
 	/* Allocate memory
 	 */
 	if (alias == NULL) {
-		length = strlen(session->host->website_root);
+		length = session->host->website_root_len;
 	} else {
 		length = strlen(alias->value);
 	}
-	length += strlen(session->uri) + strlen(session->host->start_file);
+	length += session->uri_len + strlen(session->host->start_file);
 	if ((session->file_on_disk = (char*)malloc(length + 4)) == NULL) { // + 3 for '.gz' (gzip encoding)
 		return 500;
 	}
@@ -617,7 +731,7 @@ int uri_to_path(t_session *session) {
 	/* Copy stuff
 	 */
 	if (alias == NULL) {
-		length = strlen(session->host->website_root);
+		length = session->host->website_root_len;
 		memcpy(session->file_on_disk, session->host->website_root, length);
 		strstart = session->uri;
 		if (session->local_user != NULL) {
@@ -652,7 +766,8 @@ t_access allow_client(t_session *session) {
 }
 
 int get_path_info(t_session *session) {
-	char *uri, *dot, *slash;
+	char *uri, *dot, *slash, *old_cgi_handler;
+	t_cgi_type cgi_type, old_cgi_type;
 	int ofs = 0;
 	int len;
 
@@ -660,7 +775,7 @@ int get_path_info(t_session *session) {
 		return 0;
 	}
 
-	len = strlen(session->host->website_root);
+	len = session->host->website_root_len;
 	if (len >= strlen(session->file_on_disk)) {
 		return -1;
 	}
@@ -674,16 +789,23 @@ int get_path_info(t_session *session) {
 		return 0;
 	}
 
+	old_cgi_handler = session->cgi_handler;
+	old_cgi_type = session->cgi_type;
+
 	*slash = '\0';
-	check_target_is_cgi(session);
+	cgi_type = check_target_is_cgi(session);
 	*slash = '/';
-	if (session->cgi_type != none) {
+
+	session->cgi_handler = old_cgi_handler;
+	session->cgi_type = old_cgi_type;
+
+	if (cgi_type != none) {
 		if ((session->path_info = strdup(slash)) == NULL) {
 			return -1;
 		}
 
 		ofs = slash - uri;
-		if (ofs < strlen(session->uri)) {
+		if (ofs < session->uri_len) {
 			*(session->uri + ofs) = '\0';
 		} else {
 			return -1;
@@ -700,9 +822,13 @@ int get_path_info(t_session *session) {
  */
 int serve_client(t_session *session) {
 	int result, length;
-	char *search, *hostname, *conffile;
+	char *search, *conffile, *qmark;
 	t_host *host_record;
-	bool put_method;
+	bool file_not_found = false;
+#ifdef HAVE_REWRITE
+	int i;
+	char *new_uri;
+#endif
 
 	if ((result = fetch_request(session)) != 200) {
 		return result;
@@ -710,15 +836,46 @@ int serve_client(t_session *session) {
 		return result;
 	}
 
-	put_method = (strcmp(session->method, "PUT") == 0);
-
-	session->host = session->config->first_host;
-	if ((hostname = get_headerfield("Host:", session->headerfields)) != NULL) {
-		if ((host_record = get_hostrecord(session->config->first_host, hostname, session->binding)) != NULL) {
+	//session->host = session->config->first_host;
+	if (session->hostname != NULL) {
+		if ((host_record = get_hostrecord(session->config->first_host, session->hostname, session->binding)) != NULL) {
 			session->host = host_record;
 		}
 	}
 
+#ifdef HAVE_REWRITE
+	/* URL rewrite
+	 */
+	for (i = 0; i < session->host->rewrite_rules.size; i++) {
+		if (rewrite_url(session->uri, session->host->rewrite_rules.item[i], session->config->url_rewrite, &new_uri) == -1) {
+			return 500;
+		}
+		if (new_uri != NULL) {
+			if (register_tempdata(&(session->tempdata), new_uri, tc_data) == -1) {
+				free(new_uri);
+				return 500;
+			}
+#ifdef DEBUG
+			fprintf(stderr, "Rewrite URL: [%s] --> [%s]\n", session->uri, new_uri);
+#endif
+			session->uri = new_uri;
+		}
+	}
+#endif
+
+	/* Find GET data
+	 */
+	qmark = session->uri;
+	while (*qmark != '\0') {
+		if (*qmark == '?') {
+			*qmark = '\0';
+			session->vars = qmark + 1;
+			break;
+		}
+		qmark++;
+	}
+
+	session->uri_len = strlen(session->uri);
 	url_decode(session->uri);
 	if ((session->vars != NULL) && (session->host->secure_url)) {
 		remove_dangerous_chars(session->vars);
@@ -735,7 +892,7 @@ int serve_client(t_session *session) {
 	}
 
 	if (valid_uri(session->uri) == false) {
-		if (put_method) {
+		if (session->put_method) {
 			if (session->binding->enable_alter) {
 				return 403;
 			}
@@ -791,6 +948,19 @@ int serve_client(t_session *session) {
 		return result;
 	}
 
+	switch (allow_client(session)) {
+		case deny:
+			log_error(session, fb_accesslist);
+			return 403;
+		case allow:
+			break;
+		case pwd:
+		case unknown:
+			if (http_authentication_oke(session) == false) {
+				return 401;
+			}
+	}
+
 	switch (is_directory(session->file_on_disk)) {
 		case error:
 			return 500;
@@ -798,7 +968,7 @@ int serve_client(t_session *session) {
 			session->uri_is_dir = true;
 			break;
 		case no:
-			if ((put_method == false) && (session->host->enable_path_info)) {
+			if ((session->put_method == false) && (session->host->enable_path_info)) {
 				if (get_path_info(session) == -1) {
 					return 500;
 				}
@@ -816,22 +986,11 @@ int serve_client(t_session *session) {
 				}
 			}
 
-			if ((put_method == false) && (session->accept_gzip == false)) {
+			if ((session->put_method == false) && (session->accept_gzip == false) && (session->host->fast_cgi.size == 0)) {
 				return 404;
 			}
-	}
 
-	switch (allow_client(session)) {
-		case deny:
-			log_error(session, fb_accesslist);
-			return 403;
-		case allow:
-			break;
-		case pwd:
-		case unknown:
-			if (http_authentication_oke(session) == false) {
-				return 401;
-			}
+			file_not_found = true;
 	}
 
 	length = strlen(session->file_on_disk);
@@ -845,6 +1004,12 @@ int serve_client(t_session *session) {
 		return 301;
 	}
 
+	check_target_is_cgi(session);
+
+	if (file_not_found && (session->put_method == false) && (session->cgi_type != fastcgi)) {
+		return 404;
+	}
+
 	return handle_request(session);
 }
 	
@@ -859,7 +1024,7 @@ void handle_request_result(t_session *session, int result) {
 
 	switch (result) {
 		case fe_MAX_REQUESTSIZE:
-			log_garbage(session);
+			log_error(session, "Maximum request size reached");
 			send_code(session, 413);
 			if ((session->config->ban_on_max_request_size > 0) && ip_allowed(session->ip_address, session->config->banlist_mask)) {
 				ban_ip(session->ip_address, session->config->ban_on_max_request_size, session->config->kick_on_ban);
@@ -899,6 +1064,8 @@ void handle_request_result(t_session *session, int result) {
 				ban_ip(session->ip_address, session->config->ban_on_cmdi, session->config->kick_on_ban);
 				log_system(session, "Client banned because of command injection");
 			}
+			send_code(session, 400);
+			break;
 		case rr_SQL_INJECTION:
 			log_request(session, result);
 			if (ip_allowed(session->ip_address, session->config->banlist_mask)) {
@@ -909,13 +1076,21 @@ void handle_request_result(t_session *session, int result) {
 			break;
 		case 200:
 			break;
+		case 201:
 		case 204:
 		case 304:
 		case 412:
-		case 413:
 			if (session->data_sent == false) {
 				send_header(session, result);
-				send_buffer(session, "\r\n", 2);
+				send_buffer(session, "Content-Length: 0\r\n\r\n", 21);
+			}
+			break;
+		case 411:
+		case 413:
+			session->keep_alive = false;
+			if (session->data_sent == false) {
+				send_header(session, result);
+				send_buffer(session, "Content-Length: 0\r\n\r\n", 21);
 			}
 			break;
 		case 400:
@@ -936,17 +1111,13 @@ void handle_request_result(t_session *session, int result) {
 		case 501:
 		case 503:
 			if (session->data_sent == false) {
-				result = handle_error(session, result);
-				switch (result) {
+				switch (handle_error(session, result)) {
 					case -1:
 						session->keep_alive = false;
 						break;
 					case 200:
 						break;
 					default:
-						if (result == 500) {
-							session->keep_alive = false;
-						}
 						if (session->data_sent == false) {
 							if (send_code(session, result) == -1) {
 								session->keep_alive = false;
@@ -1207,6 +1378,8 @@ int run_server(t_settings *settings) {
 	t_config           *config;
     fd_set             read_fds;
 	struct timeval     select_timeout;
+	struct stat        status;
+	mode_t             access_rights;
 
 	signal(SIGPIPE, SIG_IGN);
 	signal(SIGCHLD, SIG_IGN);
@@ -1273,6 +1446,40 @@ int run_server(t_settings *settings) {
 	}
 #endif
 
+	/* The upload directory for PUT requests
+	 */
+	if (config->create_upload_dir) {
+		if (mkdir(config->upload_directory, 0733) == -1) {
+			if (errno != EEXIST) {
+				fprintf(stderr, "Error while creating UploadDirectory '%s'\n", config->upload_directory);
+				return EXIT_FAILURE;
+			}
+		}
+		if (stat(config->upload_directory, &status) == -1) {
+			perror("stat(UploadDirectory)");
+			return EXIT_FAILURE;
+		}
+
+		access_rights = 01733;
+		if (status.st_uid != 0) {
+			if (getuid() == 0) {
+				if (chown(config->upload_directory, 0, 0) == -1) {
+					perror("chown(UploadDirectory, 0, 0)");
+					return EXIT_FAILURE;
+				}
+			} else {
+				access_rights = 01333;
+			}
+		}
+
+		if ((status.st_mode & 07777) != access_rights) {
+			if (chmod(config->upload_directory, access_rights) == -1) {
+				fprintf(stderr, "Can't change access permissions of UploadDirectory '%s'.\n", config->upload_directory);
+				return EXIT_FAILURE;
+			}
+		}
+	}
+
 	touch_logfiles(config, config->server_root);
 	tzset();
 	clearenv();
