@@ -45,7 +45,7 @@ char *make_path(char *dir, char *file) {
 	return path;
 }
 
-static enum t_fsbool outside_webroot(char *symlink, char *webroot) {
+static t_fsbool outside_webroot(char *symlink, char *webroot) {
 	char filename[257], *slash;
 	int size, count;
 
@@ -92,8 +92,8 @@ static enum t_fsbool outside_webroot(char *symlink, char *webroot) {
 	return yes;
 }
 
-enum t_fsbool contains_not_allowed_symlink(char *filename, char *webroot) {
-	enum t_fsbool contains = no, outside;
+t_fsbool contains_not_allowed_symlink(char *filename, char *webroot) {
+	t_fsbool contains = no, outside;
 	struct stat status;
 	char *slash;
 
@@ -127,7 +127,7 @@ enum t_fsbool contains_not_allowed_symlink(char *filename, char *webroot) {
 
 /* Check whether a file is directory or not.
  */
-enum t_fsbool is_directory(char *file) {
+t_fsbool is_directory(char *file) {
 	DIR *dp;
 		
 	if ((dp = opendir(file)) != NULL) {
@@ -148,7 +148,7 @@ enum t_fsbool is_directory(char *file) {
 
 /* Check whether a file can be executed or not.
  */
-enum t_fsbool can_execute(char *file, uid_t uid, gid_t gid, t_groups *groups) {
+t_fsbool can_execute(char *file, uid_t uid, gid_t gid, t_groups *groups) {
 	struct stat status;
 	gid_t *group;
 	int num = 0;
@@ -272,6 +272,9 @@ static int parse_datestr(char *org_datestr, struct tm *date) {
 				}
 
 				*(datestr + 7) = *(datestr + 11) = *(datestr + 16) = *(datestr + 19) = *(datestr + 22) = *(datestr + 25) = '\0';
+				if ((*datestr + 5) == ' ') {
+					*(datestr + 5) = '0';
+				}
 
 				if ((date->tm_mday = str2int(datestr + 5)) <= 0) {
 					break;
@@ -458,3 +461,33 @@ void remove_filelist(t_filelist *filelist) {
 		free(file);
 	}
 }
+
+#ifdef CYGWIN
+char *cygwin_to_windows(char *path) {
+	char *slash;
+
+	if (strncmp(path, "/cygdrive/", 10) != 0) {
+		return path;
+	}
+	if (*(path + 10) == '\0') {
+		return path;
+	}
+	if (*(path + 11) != '/') {
+		return path;
+	}
+
+	path += 9;
+	*path = *(path + 1);
+	*(path + 1) = ':';
+
+	slash = path;
+	while (*slash != '\0') {
+		if (*slash == '/') {
+			*slash = '\\';
+		}
+		slash++;
+	}
+
+	return path;
+}
+#endif
