@@ -5,7 +5,7 @@
 #include "libstr.h"
 #include "liblist.h"
 
-void sfree(void* ptr) {
+void sfree(void *ptr) {
 	if (ptr != NULL) {
 		free(ptr);
 	}
@@ -64,32 +64,42 @@ void init_charlist(t_charlist *list) {
 
 int parse_charlist(char *value, t_charlist *list) {
 	char *scan;
-	int i;
+	int add = 1, i;
 
-	if (list->size == 0) {
-		scan = value;
-		list->size = 1;
-		while (*scan != '\0') {
-			if (*scan == ',') {
-				*scan = '\0';
-				list->size++;
-			}
-			scan++;
-		}
-		
-		if ((list->item = (char**)malloc(list->size * sizeof(char*))) != NULL) {
-			for (i = 0; i < list->size; i++) {
-				if ((*(list->item + i) = strdup(remove_spaces(value))) == NULL) {
-					return -1;
-				}
-				value = value + strlen(value) + 1;
-			}
-
-			return 0;
+	scan = value;
+	while (*scan != '\0') {
+		if (*scan == ',') {
+			*scan = '\0';
+			add++;
 		}
+		scan++;
 	}
 
-	return -1;
+	if ((list->item = (char**)realloc(list->item, (list->size + add) * sizeof(char*))) == NULL) {
+		return -1;
+	}
+
+	for (i = 0; i < add; i++) {
+		*(list->item + list->size + i) = NULL;
+	}
+	
+	for (i = 0; i < add; i++) {
+		if ((*(list->item + list->size + i) = strdup(remove_spaces(value))) == NULL) {
+			remove_charlist(list);
+			init_charlist(list);
+			return -1;
+		}
+		value = value + strlen(value) + 1;
+	}
+
+	list->size += add;
+
+	return 0;
+}
+
+void copy_charlist(t_charlist *dest, t_charlist *src) {
+	dest->size = src->size;
+	dest->item = src->item;
 }
 
 bool in_charlist(char *item, t_charlist *list) {
@@ -106,6 +116,16 @@ bool in_charlist(char *item, t_charlist *list) {
 	return false;
 }
 
+void print_charlist(t_charlist *list) {
+	int i;
+
+	printf("print_charlist()::start\n");
+	for (i = 0; i < list->size; i++) {
+		printf("[%s]\n", *(list->item + i));
+	}
+	printf("print_charlist()::stop\n");
+}
+
 void remove_charlist(t_charlist *list) {
 	if (list->size > 0) {
 		do {
@@ -113,12 +133,13 @@ void remove_charlist(t_charlist *list) {
 			sfree(*(list->item + list->size));
 		} while (list->size > 0);
 		sfree(list->item);
+		list->item = NULL;
 	}
 }
 
 // --< accesslist >------------------------------------------------------------
 
-/* Parse an list of access levels.
+/* Parse a list of access levels.
  */
 t_accesslist *parse_accesslist(char *line, bool pwd_allowed, t_accesslist *list) {
 	t_accesslist *new = NULL;
@@ -230,7 +251,7 @@ t_access ip_allowed(t_ipv4 ip, t_accesslist *list) {
 
 // --< key/valuelist >---------------------------------------------------------
 
-/* Parse an list of key/value combinations.
+/* Parse a list of key/value combinations.
  */
 int parse_keyvaluelist(char *line, t_keyvalue **kvlist, char *seperator) {
 	char *value;
@@ -249,9 +270,12 @@ int parse_keyvaluelist(char *line, t_keyvalue **kvlist, char *seperator) {
 
 			(*kvlist)->value = NULL;
 			if (((*kvlist)->key = strdup(remove_spaces(line))) == NULL) {
+				free(*kvlist);
 				return -1;
 			}
 			if (((*kvlist)->value = strdup(remove_spaces(value))) == NULL) {
+				free((*kvlist)->key);
+				free(*kvlist);
 				return -1;
 			}
 		} else {
@@ -279,47 +303,46 @@ t_keyvalue *remove_keyvaluelist(t_keyvalue *list) {
 
 // --< temporary data >-------------------------------------------------------
 
-int register_tempdata(t_tempdata **tempdata, void *data, int type) {
-	t_tempdata *conf;
+int register_tempdata(t_tempdata **tempdata, void *data, t_tempdata_type type) {
+	t_tempdata *tdata;
 
-	if (tempdata == NULL) {
-		return 1;
+	if ((tempdata != NULL) && (data != NULL)) {
+		if ((tdata = (t_tempdata*)malloc(sizeof(t_tempdata))) == NULL) {
+			return -1;
+		}
+
+		tdata->content = data;
+		tdata->type = type;
+		tdata->next = *tempdata;
+		*tempdata = tdata;
 	}
 
-	if ((conf = (t_tempdata*)malloc(sizeof(t_tempdata))) != NULL) {
-		conf->content = data;
-		conf->type = type;
-		conf->next = *tempdata;
-		*tempdata = conf;
-
-		return 0;
-	}
-
-	return -1;
+	return 0;
 }
 
 void remove_tempdata(t_tempdata *tempdata) {
-	t_tempdata *conf;
+	t_tempdata *tdata;
 
 	while (tempdata != NULL) {
-		conf = tempdata;
+		tdata = tempdata;
 		tempdata = tempdata->next;
 
-		switch (conf->type) {
-			case tc_DATA:
-				sfree(conf->content);
+		switch (tdata->type) {
+			case tc_data:
+				sfree(tdata->content);
 				break;
-			case tc_ACCESSLIST:
-				remove_accesslist(conf->content);
+			case tc_accesslist:
+				remove_accesslist((t_accesslist*)tdata->content);
 				break;
-			case tc_KEYVALUE:
-				free(((t_keyvalue*)(conf->content))->key);
-				free(((t_keyvalue*)(conf->content))->value);
-				free(conf->content);
+			case tc_keyvalue:
+				free(((t_keyvalue*)(tdata->content))->key);
+				free(((t_keyvalue*)(tdata->content))->value);
+				free(tdata->content);
+				break;
+			case tc_charlist:
+				remove_charlist((t_charlist*)tdata->content);
 				break;
-			case tc_CHARLIST:
-				remove_charlist(conf->content);
 		}
-		free(conf);
+		free(tdata);
 	}
 }
