@@ -4,12 +4,8 @@
 #include <stdbool.h>
 #include "global.h"
 
-#define tc_DATA       0
-#define tc_CHARLIST   1
-#define tc_ACCESSLIST 2
-#define tc_KEYVALUE   3
-
 typedef enum { deny, allow, pwd, unknown } t_access;
+typedef enum { tc_data, tc_charlist, tc_accesslist, tc_keyvalue } t_tempdata_type;
 
 typedef struct type_charlist {
 	int  size;
@@ -40,7 +36,7 @@ typedef struct type_denybotlist {
 
 typedef struct type_tempdata {
 	void *content;
-	int type;
+	t_tempdata_type type;
 
 	struct type_tempdata *next;
 } t_tempdata;
@@ -49,6 +45,7 @@ void sfree(void* ptr);
 
 void init_charlist(t_charlist *list);
 int parse_charlist(char *value, t_charlist *list);
+void copy_charlist(t_charlist *dest, t_charlist *src);
 bool in_charlist(char *item, t_charlist *list);
 void remove_charlist(t_charlist *list);
 
@@ -60,7 +57,7 @@ t_access ip_allowed(t_ipv4 ip, t_accesslist *list);
 int parse_keyvaluelist(char *line, t_keyvalue **kvlist, char *seperator);
 t_keyvalue *remove_keyvaluelist(t_keyvalue *list);
 
-int register_tempdata(t_tempdata **tempdata, void *data, int type);
+int register_tempdata(t_tempdata **tempdata, void *data, t_tempdata_type type);
 void remove_tempdata(t_tempdata *tempdata);
 
 #endif
