@@ -1,9 +1,6 @@
 #ifndef _LIBSSL_H
 #define _LIBSSL_H
 
-#include "config.h"
-
-#ifdef HAVE_SSL
 #include <stdbool.h>
 #include <openssl/bio.h>
 #include <openssl/ssl.h>
@@ -18,5 +15,3 @@ void ssl_cleanup(SSL_CTX *sslContext);
 int  get_client_certificate(SSL *ssl_data, int size, char *subject, char *issuer);
 
 #endif
-
-#endif
