@@ -334,8 +334,8 @@ const char *http_error(int code) {
 		{102, "Processing"},
 */
 		{200, "OK"},
-/*
 		{201, "Created"},
+/*
 		{202, "Accepted"},
 		{203, "Non-Authoritative Information"},
 */
@@ -374,8 +374,8 @@ const char *http_error(int code) {
 /*
 		{409, "Conflict"},
 		{410, "Gone"},
-		{411, "Length Required"},
 */
+		{411, "Length Required"},
 		{412, "Precondition Failed"},
 		{413, "Request Entity Too Large"},
 /*
@@ -487,27 +487,48 @@ char *file_extension(char *filename) {
 	return NULL;
 }
 
-char *str_replace(char *line, char *from, char *to) {
-	char *pos, *new = NULL;
-	bool replaced = false;
+int str_replace(char *src, char *from, char *to, char **dst) {
+	char *pos;
+	int replaced = 0;
 
-	if ((line != NULL) && (from != NULL) && (to != NULL)) {
-		while ((pos = strstr(line, from)) != NULL) {
-			if ((new = (char*)malloc(strlen(line) - strlen(from) + strlen(to) + 1)) == NULL) {
-				return NULL;
+	*dst = NULL;
+	if ((src != NULL) && (from != NULL) && (to != NULL)) {
+		while ((pos = strstr(src, from)) != NULL) {
+			if ((*dst = (char*)malloc(strlen(src) - strlen(from) + strlen(to) + 1)) == NULL) {
+				if (*dst != NULL) {
+					free(*dst);
+				}
+				return -1;
 			}
 
 			*pos = '\0';
-			sprintf(new, "%s%s%s", line, to, pos + strlen(from));
+			sprintf(*dst, "%s%s%s", src, to, pos + strlen(from));
 
-			if (replaced) {
-				free(line);
-			} else {
-				replaced = true;
+			if (replaced > 0) {
+				free(src);
 			}
-			line = new;
+			if (replaced++ == 1000) {
+				if (*dst != NULL) {
+					free(*dst);
+				}
+				return -1;
+			}
+			src = *dst;
 		}
 	}
 
-	return new;
+	return replaced;
+}
+
+bool min_strlen(char *str, int n) {
+	int i = 0;
+
+	while (i < n) {
+		if (*(str + i) == '\0') {
+			return false;
+		}
+		i++;
+	}
+
+	return true;
 }
