@@ -20,6 +20,7 @@ int prevent_XSS(char *line);
 const char *http_error(int code);
 bool decode_base64(char *base64);
 char *file_extension(char *filename);
-char *str_replace(char *line, char *from, char *to);
+int str_replace(char *src, char *from, char *to, char **dst);
+bool min_strlen(char *str, int n);
 
 #endif
