@@ -102,6 +102,20 @@ void log_system(t_session *session, char *mesg) {
 	}
 }
 
+/* Log an error
+ */
+void log_error(t_session *session, char *mesg) {
+	FILE *fp;
+	char str[TIMESTAMP_SIZE + IP_ADDRESS_SIZE];
+
+	if ((fp = fopen(session->host->error_logfile, "a")) != NULL) {
+		print_timestamp(str);
+		print_ip_address(str + strlen(str), session->ip_address);
+		fprintf(fp, "%s%s\n", str, mesg);
+		fclose(fp);
+	}
+}
+
 /* Log a HTTP request.
  */
 void log_request(t_session *session, int code) {
@@ -120,6 +134,8 @@ void log_request(t_session *session, int code) {
 			 */
 			if ((user = session->remote_user) == NULL) {
 				user = "";
+			} else {
+				user = secure_string(user);
 			}
 
 			print_ip_address(str, session->ip_address);
@@ -127,7 +143,7 @@ void log_request(t_session *session, int code) {
 			print_timestamp(str + offset);
 			offset += strlen(str + offset);
 
-			snprintf(str + offset, BUFFER_SIZE - offset, "%d|%lld|%s|%s %s", code, (long long)session->bytes_sent, secure_string(user), secure_string(session->method), secure_string(session->uri));
+			snprintf(str + offset, BUFFER_SIZE - offset, "%d|%lld|%s|%s %s", code, (long long)session->bytes_sent, user, secure_string(session->method), secure_string(session->uri));
 			offset += strlen(str + offset);
 
 			if ((offset < BUFFER_SIZE) && (session->path_info != NULL)) {
@@ -171,16 +187,22 @@ void log_request(t_session *session, int code) {
 
 			if ((user = session->remote_user) == NULL) {
 				user = "-";
+			} else {
+				user = secure_string(user);
 			}
 
-			snprintf(str, BUFFER_SIZE, "%s - %s [%s] \"%s %s", ip_address, user, timestamp, session->method, session->uri);
+			snprintf(str, BUFFER_SIZE, "%s - %s [%s] \"%s %s", ip_address, user, timestamp, secure_string(session->method), secure_string(session->uri));
 			offset = strlen(str);
+			if ((offset < BUFFER_SIZE) && (session->path_info != NULL)) {
+				snprintf(str + offset, BUFFER_SIZE - offset, "/%s", secure_string(session->path_info));
+				offset += strlen(str + offset);
+			}
 			if ((offset < BUFFER_SIZE) && (session->vars != NULL)) {
-				snprintf(str + offset, BUFFER_SIZE - offset, "?%s", session->vars);
+				snprintf(str + offset, BUFFER_SIZE - offset, "?%s", secure_string(session->vars));
 				offset += strlen(str + offset);
 			}
 			if (offset < BUFFER_SIZE) {
-				snprintf(str + offset, BUFFER_SIZE - offset, " %s\" %d %lld", session->http_version, code, (long long)session->bytes_sent);
+				snprintf(str + offset, BUFFER_SIZE - offset, " %s\" %d %lld", secure_string(session->http_version), code, (long long)session->bytes_sent);
 			}
 
 			if (session->config->log_format == extended) {
@@ -189,7 +211,7 @@ void log_request(t_session *session, int code) {
 				offset += strlen(str + offset);
 				if (offset < BUFFER_SIZE) {
 					if ((field = get_headerfield("Referer:", session->headerfields)) != NULL) {
-						snprintf(str + offset, BUFFER_SIZE - offset, " \"%s\"", field);
+						snprintf(str + offset, BUFFER_SIZE - offset, " \"%s\"", secure_string(field));
 					} else {
 						snprintf(str + offset, BUFFER_SIZE - offset, " -");
 					}
@@ -197,7 +219,7 @@ void log_request(t_session *session, int code) {
 				}
 				if (offset < BUFFER_SIZE) {
 					if ((field = get_headerfield("User-Agent:", session->headerfields)) != NULL) {
-						snprintf(str + offset, BUFFER_SIZE - offset, " \"%s\"", field);
+						snprintf(str + offset, BUFFER_SIZE - offset, " \"%s\"", secure_string(field));
 					} else {
 						snprintf(str + offset, BUFFER_SIZE - offset, " -");
 					}
@@ -277,20 +299,6 @@ void log_file_error(t_session *session, char *file, char *mesg) {
 	}
 }
 
-/* Log an error
- */
-void log_error(t_session *session, char *mesg) {
-	FILE *fp;
-	char str[TIMESTAMP_SIZE + IP_ADDRESS_SIZE];
-
-	if ((fp = fopen(session->host->error_logfile, "a")) != NULL) {
-		print_timestamp(str);
-		print_ip_address(str + strlen(str), session->ip_address);
-		fprintf(fp, "%s%s\n", str, mesg);
-		fclose(fp);
-	}
-}
-
 /* Close open accesslogfiles.
  */
 void log_close(t_host *host) {
