@@ -91,6 +91,10 @@ int parse_line(t_config *config, char *line) {
 			if ((config->forks = strdup(value)) == NULL) {
 				return -1;
 			}
+		} else if (strcmp(key, "setenv") == 0) {
+			if (parse_keyvaluelist(value, &(config->envir), "=") != -1) {
+				return true;
+			}
 		} else if (strcmp(key, "server") == 0) {
 			if ((new = (t_php_fcgi*)malloc(sizeof(t_php_fcgi))) == NULL) {
 				return -1;
@@ -169,14 +173,6 @@ int parse_line(t_config *config, char *line) {
 		} else {
 			return -1;
 		}
-	} else {
-		if (strcmp(key, "setenvir") == 0) {
-			if (parse_keyvaluelist(value, &(config->envir), "=") != -1) {
-				return true;
-			}
-		} else {
-			return -1;
-		}
 	}
 
 	return 0;
