@@ -24,6 +24,7 @@
 #endif
 
 #define MAX_CHUNK_SIZE 2048
+#define MAX_TO_BUFFER 400
 #define NONCE_DIGITS 10
 
 char *hs_http10 = "HTTP/1.0 ";                  //  9
@@ -119,7 +120,7 @@ static int send_to_client(t_session *session, const char *buffer, int size) {
 /* This function has been added to improve speed by buffering small amounts of data to be sent.
  */
 int send_buffer(t_session *session, const char *buffer, int size) {
-	if (size > 400) {
+	if (size > MAX_TO_BUFFER) {
 		if (session->output_size > 0) {
 			if (send_to_client(session, session->output_buffer, session->output_size) == -1) {
 				return -1;
@@ -270,7 +271,7 @@ static int send_chunk_to_client(t_session *session, const char *chunk, int size)
 }
 
 int send_chunk(t_session *session, const char *chunk, int size) {
-	if (size > 400) {
+	if (size > MAX_TO_BUFFER) {
 		if (session->output_size > 0) {
 			if (send_chunk_to_client(session, session->output_buffer, session->output_size) == -1) {
 				return -1;
@@ -359,7 +360,7 @@ int send_code(t_session *session, int code) {
 					return -1;
 				}
 			}
-			if (send_buffer(session, session->uri, strlen(session->uri)) == -1) {
+			if (send_buffer(session, session->uri, session->uri_len) == -1) {
 				return -1;
 			}
 #ifdef HAVE_SSL
@@ -589,3 +590,22 @@ void send_digest_auth(t_session *session) {
 	}
 	send_buffer(session, "\", algorithm=MD5, stale=false\r\n", 31);
 }
+
+int write_buffer(int handle, const char *buffer, int size) {
+	long total_written = 0;
+	ssize_t bytes_written;
+
+	if (size <= 0) {
+		return 0;
+	} else while (total_written < size) {
+		if ((bytes_written = write(handle, buffer + total_written, size - total_written)) == -1) {
+			if (errno != EINTR) {
+				return -1;
+			}
+		} else {
+			total_written += bytes_written;
+		}
+	}
+
+	return 0;
+}
