@@ -19,8 +19,13 @@
 #define ID_NOBODY             65534
 #define MAX_LENGTH_CONFIGLINE   512
 #define MAX_CACHE_SIZE           50
+#define MAX_UPLOAD_SIZE         100
 
-enum t_section { syntax_error = -1, none, binding, virtual_host, directory, fcgi_server };
+enum t_section { syntax_error = -1, none, binding, virtual_host, directory, fcgi_server
+#ifdef HAVE_REWRITE
+	, url_rewrite
+#endif
+	};
 
 bool including = false;
 t_keyvalue *variables = NULL;
@@ -36,6 +41,7 @@ static t_host *new_host(void) {
 
 	if ((host = (t_host*)malloc(sizeof(t_host))) != NULL) {
 		host->website_root       = NULL;
+		host->website_root_len   = 0;
 		host->start_file         = "index.html";
 		host->error_401_handler  = NULL;
 		host->error_403_handler  = NULL;
@@ -63,6 +69,9 @@ static t_host *new_host(void) {
 		init_charlist(&(host->required_binding));
 		init_charlist(&(host->required_group));
 		init_charlist(&(host->alter_group));
+#ifdef HAVE_REWRITE
+		init_charlist(&(host->rewrite_rules));
+#endif
 		host->wrap_cgi           = NULL;
 		host->groups.number      = 0;
 		host->groups.array       = NULL;
@@ -163,9 +172,10 @@ static t_binding *new_binding(void) {
 		binding->binding_id           = NULL;
 		binding->socket               = -1;
 		binding->enable_trace         = false;
-		binding->enable_trace         = false;
+		binding->enable_alter         = false;
 		binding->max_keepalive        = 50;
 		binding->max_request_size     = 65536;
+		binding->max_upload_size      = 1048576;
 		binding->time_for_1st_request = 5;
 		binding->time_for_request     = 30;
 
@@ -178,6 +188,24 @@ static t_binding *new_binding(void) {
 	return binding;
 }
 
+#ifdef HAVE_REWRITE
+t_url_rewrite *new_url_rewrite(void) {
+	t_url_rewrite *url_rewrite;
+
+	if ((url_rewrite = (t_url_rewrite*)malloc(sizeof(t_url_rewrite))) != NULL) {
+		url_rewrite->rewrite_id = NULL;
+		url_rewrite->rewrite_rule = NULL;
+
+		url_rewrite->next = NULL;
+	} else {
+		perror("new_url_rewrite()");
+		exit(EXIT_FAILURE);
+	}
+
+	return url_rewrite;
+}
+#endif
+
 t_config *default_config(void) {
 	t_config *config;
 
@@ -237,6 +265,11 @@ t_config *default_config(void) {
 		config->reconnect_delay    = 0;
 		config->banlist_mask       = NULL;
 		config->fcgi_server        = NULL;
+		config->upload_directory   = UPLOAD_DIR;
+		config->create_upload_dir  = false;
+#ifdef HAVE_REWRITE
+		config->url_rewrite        = NULL;
+#endif
 #ifdef CYGWIN
 		config->platform           = windows;
 #endif
@@ -388,7 +421,7 @@ bool replace_variables(char **line) {
 
 	variable = variables;
 	while (variable != NULL) {
-		if ((new = str_replace(*line, variable->key, variable->value)) != NULL) {
+		if (str_replace(*line, variable->key, variable->value, &new) > 0) {
 			if (replaced) {
 				free(*line);
 			}
@@ -415,6 +448,8 @@ int check_configuration(t_config *config) {
 #ifdef HAVE_SSL
 	t_binding *binding;
 
+	config->upload_directory_len = strlen(config->upload_directory);
+
 	binding = config->binding;
 	while (binding != NULL) {
 		if (binding->use_ssl && (binding->server_key == NULL)) {
@@ -422,6 +457,9 @@ int check_configuration(t_config *config) {
 			binding->ca_cert = config->ca_cert;
 			binding->verify_depth = config->verify_depth;
 		}
+		if (binding->enable_alter) {
+			config->create_upload_dir = true;
+		}
 		binding = binding->next;
 	}
 #endif
@@ -479,6 +517,12 @@ int check_configuration(t_config *config) {
 		fcgi_server = fcgi_server->next;
 	}
 
+#ifdef HAVE_REWRITE
+	if (rewrite_rules_oke(config->url_rewrite) == false) {
+		return -1;
+	}
+#endif
+
 	if (config->binding == NULL) {
 		fprintf(stderr, "No binding defined.\n");
 		return -1;
@@ -608,13 +652,15 @@ static bool system_setting(char *key, char *value, t_config *config) {
 		}
 	} else if (strcmp(key, "cgihandler") == 0) {
 		if (split_string(value, &value, &rest, ':') == 0) {
-			CGI = config->cgi_handler;
-			if ((config->cgi_handler = (t_cgi_handler*)malloc(sizeof(t_cgi_handler))) != NULL) {
-				config->cgi_handler->next = CGI;
-				if ((config->cgi_handler->handler = strdup(value)) != NULL) {
-					init_charlist(&(config->cgi_handler->extension));
-					if (parse_charlist(rest, &(config->cgi_handler->extension)) != -1) {
-						return true;
+			if ((*value != '\0') && (*rest != '\0')) {
+				CGI = config->cgi_handler;
+				if ((config->cgi_handler = (t_cgi_handler*)malloc(sizeof(t_cgi_handler))) != NULL) {
+					config->cgi_handler->next = CGI;
+					if ((config->cgi_handler->handler = strdup(value)) != NULL) {
+						init_charlist(&(config->cgi_handler->extension));
+						if (parse_charlist(rest, &(config->cgi_handler->extension)) != -1) {
+							return true;
+						}
 					}
 				}
 			}
@@ -775,6 +821,12 @@ static bool system_setting(char *key, char *value, t_config *config) {
 		if ((config->time_for_cgi = str2int(value)) >= 1) {
 			return true;
 		}
+	} else if (strcmp(key, "uploaddirectory") == 0) {
+		if (valid_directory(value)) {
+			if ((config->upload_directory = strdup(value)) != NULL) {
+				return true;
+			}
+		}
 	} else if (strcmp(key, "userdirectory") == 0) {
 		if ((*value != '/') && (strchr(value, '.') == NULL)) {
 			if ((config->user_directory = strdup(value)) != NULL) {
@@ -1017,6 +1069,12 @@ static bool host_setting(char *key, char *value, t_host *host) {
 		if (parse_yesno(value, &(host->require_resolve_ip)) == 0) {
 			return true;
 		}
+#ifdef HAVE_REWRITE
+	} else if (strcmp(key, "rewriteurl") == 0) {
+		if (parse_charlist(value, &(host->rewrite_rules)) == 0) {
+			return true;
+		}
+#endif
 	} else if (strcmp(key, "secureurl") == 0) {
 		if (parse_yesno(value, &(host->secure_url)) == 0) {
 			return true;
@@ -1037,6 +1095,7 @@ static bool host_setting(char *key, char *value, t_host *host) {
 	} else if (strcmp(key, "websiteroot") == 0) {
 		if (valid_directory(value)) {
 			if ((host->website_root = strdup(value)) != NULL) {
+				host->website_root_len = strlen(host->website_root);
 				return true;
 			}
 		}
@@ -1096,6 +1155,9 @@ static bool directory_setting(char *key, char *value, t_directory *directory) {
 			return true;
 		}
 	} else if (strcmp(key, "path") == 0) {
+		if (directory->path != NULL) {
+			return false;
+		}
 		length = strlen(value);
 		if ((length < 128) && (*value == '/')) {
 			if (*(value + length - 1) == '/') {
@@ -1178,9 +1240,18 @@ static bool binding_setting(char *key, char *value, t_binding *binding) {
 			binding->max_request_size <<= 10;
 			return true;
 		}
+	} else if (strcmp(key, "maxuploadsize") == 0) {
+		if ((binding->max_upload_size = str2int(value)) > 0) {
+			if (binding->max_upload_size <= MAX_UPLOAD_SIZE) {
+				binding->max_upload_size <<= 20;
+				return true;
+			}
+		}
 	} else if (strcmp(key, "bindingid") == 0) {
-		if ((binding->binding_id = strdup(value)) != NULL) {
-			return true;
+		if (binding->binding_id == NULL) {
+			if ((binding->binding_id = strdup(value)) != NULL) {
+				return true;
+			}
 		}
 	} else if (strcmp(key, "port") == 0) {
 		if ((binding->port = str2int(value)) > 0) {
@@ -1259,8 +1330,10 @@ static bool fcgi_server_setting(char *key, char *value, t_fcgi_server *fcgi_serv
 			return true;
 		}
 	} else if (strcmp(key, "fastcgiid") == 0) {
-		if ((fcgi_server->fcgi_id = strdup(value)) != NULL) {
-			return true;
+		if (fcgi_server->fcgi_id == NULL) {
+			if ((fcgi_server->fcgi_id = strdup(value)) != NULL) {
+				return true;
+			}
 		}
 	} else if (strcmp(key, "serverroot") == 0) {
 		if ((fcgi_server->chroot = strdup(value)) != NULL) {
@@ -1327,6 +1400,9 @@ int read_main_configfile(char *configfile, t_config *config, bool config_check)
 	t_directory *current_directory = NULL;
 	t_binding *current_binding = NULL;
 	t_fcgi_server *current_fcgi_server = NULL;
+#ifdef HAVE_REWRITE
+	t_url_rewrite *current_rewrite = NULL;
+#endif
 
 	/* Read and parse Hiawatha configurationfile.
 	 */
@@ -1398,6 +1474,13 @@ int read_main_configfile(char *configfile, t_config *config, bool config_check)
 					current_host->next = new_host();
 					current_host = current_host->next;
 					section = virtual_host;
+#ifdef HAVE_REWRITE
+				} else if (strcmp(key, "urlrewrite") == 0) {
+					current_rewrite = new_url_rewrite();
+					current_rewrite->next = config->url_rewrite;
+					config->url_rewrite = current_rewrite;
+					section = url_rewrite;
+#endif
 				}
 			} else if (strcmp(key, "}") == 0) {
 				/* Section end
@@ -1442,10 +1525,20 @@ int read_main_configfile(char *configfile, t_config *config, bool config_check)
 								current_host = config->first_host;
 							}
 						} else {
-							fprintf(stderr, "A Hostname is missing in %s\n", configfile);
+							fprintf(stderr, "A Hostname is missing in a VirtualHost section in %s\n", configfile);
 							retval = -1;
 						}
 						break;
+#ifdef HAVE_REWRITE
+					case url_rewrite:
+						if (current_rewrite->rewrite_id != NULL) {
+							current_rewrite = NULL;
+						} else {
+							fprintf(stderr, "A RewriteID is missing in a UrlRewrite section in %s\n", configfile);
+							retval = -1;
+						}
+						break;
+#endif
 					default:
 						retval = counter;
 				}
@@ -1511,6 +1604,12 @@ int read_main_configfile(char *configfile, t_config *config, bool config_check)
 							if (fcgi_server_setting(key, value, current_fcgi_server)) {
 								break;
 							}
+#ifdef HAVE_REWRITE
+						} else if (section == url_rewrite) {
+							if (url_rewrite_setting(key, value, current_rewrite)) {
+								break;
+							}
+#endif
 						}
 						retval = counter;
 					} while (false);
