@@ -6,6 +6,9 @@
 #include <pthread.h>
 #include "global.h"
 #include "mimetype.h"
+#ifdef HAVE_REWRITE
+#include "rewrite.h"
+#endif
 #include "libfs.h"
 #include "liblist.h"
 #include "userconfig.h"
@@ -79,6 +82,7 @@ typedef struct type_binding {
 	bool          enable_alter;
 	int           max_keepalive;
 	long          max_request_size;
+	long          max_upload_size;
 	int           time_for_1st_request;
 	int           time_for_request;
 
@@ -126,6 +130,7 @@ typedef struct type_directory {
 
 typedef struct type_host {
 	char          *website_root;
+	int           website_root_len;
 	char          *start_file;
 	char          *error_401_handler;
 	char          *error_403_handler;
@@ -161,6 +166,9 @@ typedef struct type_host {
 	char          *imgref_replacement;
 	t_keyvalue    *envir_str;
 	t_keyvalue    *alias;
+#ifdef HAVE_REWRITE
+	t_charlist    rewrite_rules;
+#endif
 #ifdef HAVE_SSL
     bool          require_ssl;
 #endif
@@ -209,7 +217,10 @@ typedef struct type_config {
 	t_host        *first_host;
 	t_directory   *directory;
 	t_throttle    *throttle;
-	t_cgi_handler  *cgi_handler;
+#ifdef HAVE_REWRITE
+	t_url_rewrite *url_rewrite;
+#endif
+	t_cgi_handler *cgi_handler;
 	char          *cgi_wrapper;
 	bool          wrap_user_cgi;
 #ifdef CYGWIN
@@ -231,6 +242,10 @@ typedef struct type_config {
 	t_accesslist  *banlist_mask;
 	t_fcgi_server *fcgi_server;
 
+	char          *upload_directory;
+	int           upload_directory_len;
+	bool          create_upload_dir;
+
 #ifdef HAVE_CACHE
 	off_t         cache_size;
 	off_t         cache_max_filesize;
