@@ -20,9 +20,6 @@
 #ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
 #endif
-#ifdef HAVE_SENDFILE
-#include <sys/sendfile.h>
-#endif
 #include "global.h"
 #include "alternative.h"
 #include "libstr.h"
@@ -54,10 +51,10 @@
 
 #define NEW_FILE -1
 
-char *chunked  = "Transfer-Encoding: chunked\r\n";
-char *hs_allow = "Allow: GET, HEAD, OPTIONS";
-char *hs_post  = ", POST";
-char *hs_trace = ", TRACE";
+char *hs_chunked = "Transfer-Encoding: chunked\r\n";
+char *hs_allow   = "Allow: GET, HEAD, OPTIONS";
+char *hs_post    = ", POST";
+char *hs_trace   = ", TRACE";
 
 extern char *fb_filesystem;
 extern char *fb_symlink;
@@ -189,7 +186,7 @@ int send_file(t_session *session) {
 
 	/* Set throttlespeed
 	 */
-	dot = session->uri + strlen(session->uri);
+	dot = session->uri + session->uri_len;
 	while ((*dot != '.') && (dot != session->uri)) {
 		dot--;
 	}
@@ -466,6 +463,12 @@ int execute_cgi(t_session *session) {
 		return 403;
 	}
 
+#ifdef CYGWIN
+	if ((session->config->platform == windows) && (session->cgi_type == program)) {
+		chmod(session->file_on_disk, 0755);
+	}
+#endif
+
 	if ((wrap_cgi == false) && (session->cgi_type != fastcgi)) {
 		if (session->cgi_type == program) {
 			switch (can_execute(session->file_on_disk, session->config->server_uid, session->config->server_gid, &(session->config->groups))) {
@@ -585,6 +588,7 @@ int execute_cgi(t_session *session) {
 		}
 	}
 
+	cgi_info.type = session->cgi_type;
 	cgi_info.input_buffer_size = cgi_info.error_buffer_size = CGI_BUFFER_SIZE;
 	cgi_info.input_len = cgi_info.error_len = 0;
 
@@ -717,6 +721,7 @@ int execute_cgi(t_session *session) {
 						*(cgi_info.input_buffer + cgi_info.input_len) = '\0';
 
 						if ((new_line = strstr(cgi_info.input_buffer, "\r\n\r\n")) != NULL) {
+							*(new_line + 2) = '\0';
 							if (session->throttle == 0) {
 								if ((contyp_start = strcasestr(cgi_info.input_buffer, hs_contyp)) != NULL) {
 									if ((contyp_end = strchr(contyp_start, '\r')) != NULL) {
@@ -735,15 +740,19 @@ int execute_cgi(t_session *session) {
 								}
 							} else if ((code = strcasestr(cgi_info.input_buffer, "Status: ")) != NULL) {
 								line = code += 8;
+
+								retval = -1;
 								while (*line != '\0') {
 									if (*line == '\r') {
 										*line = '\0';
+										retval = str2int(code);
+										*line = '\r';
 										break;
 									}
 									line++;
 								}
 
-								if ((retval = str2int(code)) <= 0) {
+								if (retval <= 0) {
 									log_file_error(session, session->file_on_disk, "invalid status code received from CGI");
 									retval = 500;
 									break;
@@ -770,10 +779,11 @@ int execute_cgi(t_session *session) {
 							}
 							if ((strcasestr(cgi_info.input_buffer, hs_conlen) != NULL) || (session->keep_alive == false)) {
 								send_in_chunks = false;
-							} else if (send_buffer(session, chunked, 28) == -1) {
+							} else if (send_buffer(session, hs_chunked, 28) == -1) {
 								retval = rs_DISCONNECT;
 								break;
 							}
+							*(new_line + 2) = '\r';
 
 							/* Send the header.
 							 */
@@ -978,7 +988,7 @@ int show_index(t_session *session) {
 
 	do {
 		if (session->keep_alive) {
-			if (send_buffer(session, chunked, 28) == -1) {
+			if (send_buffer(session, hs_chunked, 28) == -1) {
 				break;
 			}
 		} 
@@ -988,7 +998,7 @@ int show_index(t_session *session) {
 			break;
 		} else if (send_chunk(session, "<html><head><title>Index of ", 28) == -1) {
 			break;
-		} else if (send_chunk(session, session->uri, strlen(session->uri)) == -1) {
+		} else if (send_chunk(session, session->uri, session->uri_len) == -1) {
 			break;
 		} else if (send_chunk(session,	"</title>\n", 9) == -1) {
 			break;
@@ -1014,7 +1024,7 @@ int show_index(t_session *session) {
 			break;
 		} else if (send_chunk(session, " : ", 3) == -1) {
 			break;
-		} else if (send_chunk(session, session->uri, strlen(session->uri)) == -1) {
+		} else if (send_chunk(session, session->uri, session->uri_len) == -1) {
 			break;
 		} else if (send_chunk(session, "</div>\n<table class=\"list\">\n<tr><th class=\"left\"></th><th class=\"filename\">Filename</th><th class=\"date\">Date</th><th class=\"size\">Size</th><th class=\"right\"></th></tr>\n", 169) == -1) {
 			break;
@@ -1042,9 +1052,12 @@ int show_index(t_session *session) {
 					break;
 				}
 			}
+
+			len = strlen(file->name);
+
 			if (send_chunk(session, "</td><td class=\"filename\"><a href=\"", 35) == -1) {
 				break;
-			} else if (send_chunk(session, file->name, strlen(file->name)) == -1) {
+			} else if (send_chunk(session, file->name, len) == -1) {
 				break;
 			}
 			if (file->is_dir) {
@@ -1054,7 +1067,7 @@ int show_index(t_session *session) {
 			}
 			if (send_chunk(session, "\">", 2) == -1) {
 				break;
-			} else if (send_chunk(session, file->name, strlen(file->name)) == -1) {
+			} else if (send_chunk(session, file->name, len) == -1) {
 				break;
 			}
 			if (file->is_dir) {
@@ -1190,7 +1203,7 @@ int handle_trace_request(t_session *session) {
 	t_headerfield *header;
 
 	body_size = 3;
-	body_size += strlen(session->method) + strlen(session->uri);
+	body_size += strlen(session->method) + session->uri_len;
 	if (session->vars != NULL) {
 		body_size += 1 + strlen(session->vars);
 	}
@@ -1277,11 +1290,15 @@ static t_access allow_alter(t_session *session) {
 }
 
 int handle_put_request(t_session *session) {
-	int handle, bytes_written, total_written = 0, result = 204;
-	char *range, *value, *rest;
+	int handle_write, handle_read = -1, result = -1, total_written = 0;
+	off_t write_begin, write_end, total_size, file_size;
+	ssize_t bytes_read;
+	char *range, *value, *rest, *buffer;
 	bool range_found;
-	long write_begin, write_end, total_size;
-	off_t file_size;
+
+	if (session->uploaded_file == NULL) {
+		return 500;
+	}
 
 	switch (allow_alter(session)) {
 		case deny:
@@ -1307,31 +1324,34 @@ int handle_put_request(t_session *session) {
 	range = get_headerfield("Content-Range:", session->headerfields);
 	range_found = (range != NULL);
 
-	if ((handle = open(session->file_on_disk, O_WRONLY)) == -1) {
+	if ((handle_write = open(session->file_on_disk, O_WRONLY)) == -1) {
 		// New file
 		if (range_found) {
 			return 416;
 		}
-		if ((handle = open(session->file_on_disk, O_CREAT | O_WRONLY)) == -1) {
+		if ((handle_write = open(session->file_on_disk, O_CREAT | O_WRONLY)) == -1) {
 			log_file_error(session, session->file_on_disk, fb_filesystem);
 			return 403;
 		}
 		file_size = NEW_FILE;
+		result = 201;
 	} else {
 		// Existing file
 		if ((file_size = filesize(session->file_on_disk)) == -1) {
+			close(handle_write);
 			return 500;
 		}
+		result = 204;
 	}
-	if (flock(handle, LOCK_EX) == -1) {
+	if (flock(handle_write, LOCK_EX) == -1) {
 		log_file_error(session, session->file_on_disk, "can't lock file for writing");
-		close(handle);
+		close(handle_write);
 		if (file_size == NEW_FILE) {
 			unlink(session->file_on_disk);
 		}
 		return 500;
 	}
-	if (fchmod(handle, session->host->alter_fmode) == -1) {
+	if (fchmod(handle_write, session->host->alter_fmode) == -1) {
 		log_file_error(session, session->file_on_disk, "can't change file mode");
 	}
 
@@ -1343,6 +1363,8 @@ int handle_put_request(t_session *session) {
 
 			if (split_string(range, &value, &rest, '-') == -1) {
 				result = 416;
+			} else if (strlen(value) > 9) {
+				result = 416;
 			} else if ((write_begin = str2int(value)) == -1) {
 				result = 416;
 			} else if (split_string(rest, &value, &rest, '/') == -1) {
@@ -1357,10 +1379,10 @@ int handle_put_request(t_session *session) {
 				result = 416;
 			} else if (write_begin > file_size) {
 				result = 416;
-			} else if (session->content_length != (write_end - write_begin + 1)) {
+			} else if (session->uploaded_size != (write_end - write_begin + 1)) {
 				result = 416;
 			} else if (write_begin > 0) {
-				if (lseek(handle, write_begin, SEEK_SET) == -1) {
+				if (lseek(handle_write, write_begin, SEEK_SET) == -1) {
 					result = 500;
 				}
 			}
@@ -1369,25 +1391,47 @@ int handle_put_request(t_session *session) {
 		}
 	}
 
-	if (result == 204) {
-		if ((file_size != NEW_FILE) && (range_found == false)) {
-			ftruncate(handle, session->content_length);
+	if ((result == 201) || (result == 204)) {
+		if ((handle_read = open(session->uploaded_file, O_RDONLY)) == -1) {
+			close(handle_write);
+			if (file_size == NEW_FILE) {
+				unlink(session->file_on_disk);
+			}
+			return 500;
 		}
-		while (total_written < session->content_length) {
-			if ((bytes_written = write(handle, session->body + total_written, session->content_length - total_written)) == -1) {
-				if (errno != EINTR) {
+
+		if ((file_size != NEW_FILE) && (range_found == false)) {
+			ftruncate(handle_write, session->uploaded_size);
+		}
+		
+		if ((buffer = (char*)malloc(FILE_BUFFER_SIZE)) != NULL) {
+			while (total_written < session->uploaded_size) {
+				if ((bytes_read = read(handle_read, buffer, FILE_BUFFER_SIZE)) != -1) {
+					if (bytes_read == 0) {
+						break;
+					} else if (write_buffer(handle_write, buffer, bytes_read) != -1) {
+						total_written += bytes_read;
+					} else {
+						result = 500;
+						break;
+					}
+				} else if (errno != EINTR) {
 					result = 500;
 					break;
 				}
-			} else {
-				total_written += bytes_written;
 			}
+			free(buffer);
+		} else {
+			result = 500;
 		}
 	}
 
-	flock(handle, LOCK_UN);
-	close(handle);
-	if ((result != 204) && (file_size == NEW_FILE)) {
+	if (handle_read != -1) {
+		close(handle_read);
+	}
+	flock(handle_write, LOCK_UN);
+	close(handle_write);
+	if ((result != 201) && (result != 204) && (file_size == NEW_FILE)) {
 		unlink(session->file_on_disk);
 	}
 
