@@ -6,6 +6,11 @@
 #include <string.h>
 #include "libstr.h"
 #include "libfs.h"
+#ifdef HAVE_REWRITE
+#include "rewrite.h"
+#endif
+
+#define MAX_INPUT 1024
 
 typedef struct type_line {
 	char *key, *value, *file;
@@ -133,7 +138,7 @@ int read_directory(char *dir, t_line **config, t_line **aliases) {
 
 int read_file(char *config_file, t_line **config, t_line **aliases, bool handle_include) {
 	FILE *fp;
-	char line[1025], *data, *value;
+	char line[MAX_INPUT + 1], *data, *value;
 	bool is_alias;
 	int linenr = 0, retval = 0;
 
@@ -146,8 +151,8 @@ int read_file(char *config_file, t_line **config, t_line **aliases, bool handle_
 		return -1;
 	}
 
-	line[1024] = '\0';
-	while (fgets(line, 1024, fp) != NULL) {
+	line[MAX_INPUT] = '\0';
+	while (fgets(line, MAX_INPUT, fp) != NULL) {
 		linenr++;
 		data = strip_line(line);
 		if ((data[0] == '#') || (data[0] == '\0')) {
@@ -191,6 +196,35 @@ int read_file(char *config_file, t_line **config, t_line **aliases, bool handle_
 	return retval;
 }
 
+int read_config_file(char *config_file, t_line **config) {
+	t_line *aliases = NULL, *alias, *line;
+	char *new_value;
+	int retval;
+
+	if ((retval = read_file(config_file, config, &aliases, true)) == -1) {
+		return -1;
+	}
+
+	/* Replace the aliasses
+	 */
+	line = *config;
+	while (line != NULL) {
+		alias = aliases;
+		while (alias != NULL) {
+			if (line->value != NULL) {
+				if (str_replace(line->value, alias->key, alias->value, &new_value) > 0) {
+					line->value = new_value;
+				}
+			}
+			alias = alias->next;
+		}
+		line = line->next;
+	}
+	dispose_result(aliases, true);
+
+	return retval;
+}
+
 bool is_ip_address(char *str) {
 	char *digit;
 	int digits = 0, value;
@@ -209,7 +243,7 @@ bool is_ip_address(char *str) {
 
 int check_main_config(char *config_dir) {
 	int errors = 0;
-	t_line *config = NULL, *aliases = NULL, *alias, *haystack, *needles, *needle;
+	t_line *config = NULL, *haystack, *needles, *needle;
 	char *item, *rest, *info;
 	bool inside_section, has_dot;
 
@@ -225,27 +259,10 @@ int check_main_config(char *config_dir) {
 	/* Read the configuration file
 	 */
 	config = NULL;
-	if (read_file(strdup("httpd.conf"), &config, &aliases, true) != 0) {
+	if (read_config_file(strdup("httpd.conf"), &config) != 0) {
 		return 1;
 	}
 
-	/* Replace the aliasses
-	 */
-	needle = config;
-	while (needle != NULL) {
-		alias = aliases;
-		while (alias != NULL) {
-			if (needle->value != NULL) {
-				if ((info = str_replace(needle->value, alias->key, alias->value)) != NULL) {
-					needle->value = info;
-				}
-			}
-			alias = alias->next;
-		}
-		needle = needle->next;
-	}
-	dispose_result(aliases, true);
-
 	/* FastCGI Id check
 	 */
 	haystack = search_key(config, "fastcgiid");
@@ -372,11 +389,114 @@ int check_main_config(char *config_dir) {
 	return errors;
 }
 
+#ifdef HAVE_REWRITE
+void check_url_rewrite(char *config_dir, char **rewrite_id) {
+	t_line *config = NULL;
+	char input[MAX_INPUT + 1], **id, *url, *new_url;
+	t_url_rewrite *url_rewrite, *rewrite = NULL;
+	bool in_rule_section = false;
+
+	if (chdir(config_dir) == -1) {
+		perror(config_dir);
+		return;
+	}
+
+	/* Read the configuration file
+	 */
+	config = NULL;
+	if (read_config_file(strdup("httpd.conf"), &config) != 0) {
+		return;
+	}
+
+	/* Parse the URL rewrite rules
+	 */
+	url_rewrite = NULL;
+	while (config != NULL) {
+		if (in_rule_section) {
+			if ((strcmp(config->key, "}") == 0) && (strcmp(config->value, "") == 0)) {
+				in_rule_section = false;
+			} else if (url_rewrite_setting(config->key, config->value, rewrite) == false) {
+				fprintf(stderr, "UrlRewrite error in %s on line %d.\n", config->file, config->linenr);
+				return;
+			}
+		} else if ((strcmp(config->key, "urlrewrite") == 0) && (strcmp(config->value, "{") == 0)) {
+			if (url_rewrite == NULL) {
+				url_rewrite = rewrite = (t_url_rewrite*)malloc(sizeof(t_url_rewrite));
+			} else {
+				rewrite->next = (t_url_rewrite*)malloc(sizeof(t_url_rewrite));
+				rewrite = rewrite->next;
+			}
+			rewrite->rewrite_id = NULL;
+			rewrite->rewrite_rule = NULL;
+			rewrite->next = NULL;
+
+			in_rule_section = true;
+		}
+		config = config->next;
+	}
+
+	if (url_rewrite == NULL) {
+		printf("No URL rewrite rules found.\n");
+		return;
+	}
+
+	if (rewrite_rules_oke(url_rewrite) == false) {
+		return;
+	}
+
+	if (*rewrite_id == NULL) {
+		printf("No errors found in URL rewrite rules.\n");
+		return;
+	} else
+
+	id = rewrite_id;
+	do {
+		if (find_url_rewrite(*id, url_rewrite) == false) {
+			printf("RewriteID '%s' not found.\n", *id);
+			return;
+		}
+		id++;
+	} while (*id != NULL);
+
+	/* Start testing
+	 */
+	input[MAX_INPUT] = '\0';
+	printf("\nType 'exit' to leave the program.\n\nurl: ");
+	while (fgets(input, MAX_INPUT, stdin) != NULL) {
+		url = remove_spaces(input);
+
+		if ((strcmp(url, "exit") == 0) || (strcmp(url, "quit") == 0)) {
+			break;
+		}
+
+		id = rewrite_id;
+		while (*id != NULL) {
+			rewrite_url(url, *id, url_rewrite, &new_url);
+			if (new_url != NULL) {
+				url = new_url;
+			}
+			id++;
+		}
+
+		if (new_url != NULL) {
+			printf("new: %s\n\n", new_url);
+		} else {
+			printf("old: %s\n\n", url);
+		}
+
+		printf("url: ");
+	}
+}
+#endif
+
 void show_help(char *wigwam) {
 	printf("Usage: %s [options]\n", wigwam);
-	printf("Options: -c path: path to where the configrationfiles are located.\n");
+	printf("Options: -c <path>: path to where the configrationfiles are located.\n");
 	printf("         -h: show this information and exit.\n");
 	printf("         -q: don't print the test results.\n");
+#ifdef HAVE_REWRITE
+	printf("         -r [<rewrite_id> ...]: test URL rewriting rule(s).\n");
+#endif
 	printf("         -v: show version and exit.\n");
 }
 
@@ -395,6 +515,11 @@ int main(int argc, char *argv[]) {
         } else if (strcmp(argv[i], "-h") == 0) {
 			show_help(argv[0]);
 			return EXIT_SUCCESS;
+#ifdef HAVE_REWRITE
+		} else if (strcmp(argv[i], "-r") == 0) {
+			check_url_rewrite(config_dir, argv + i + 1);
+			return EXIT_SUCCESS;
+#endif
 		} else if (strcmp(argv[i], "-q") == 0) {
 			quiet = true;
 		} else if (strcmp(argv[i], "-v") == 0) {
